MIT License

Copyright (c) 2022 KeitaKawase

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.

# 基本方針

YouTube 「マルバツつくってプログラミングまなぶ（下）」(あさぱん動画,確認日2022/07/11)]https://www.bing.com/ck/a?!&&p=866c075c04e9d5f2JmltdHM9MTY1NzQxNDI5MCZpZ3VpZD01YmJjN2RlYS01Mzg3LTRhOTgtOTVlYS1hYzBhN2E5ZDM3YzAmaW5zaWQ9NTE3NA&ptn=3&hsh=3&fclid=6f9cc0b3-ffea-11ec-93d8-cf37186d974c&u=a1aHR0cHM6Ly93d3cueW91dHViZS5jb20vd2F0Y2g_dj0zOFFNUnRweVpGdw&ntb=1
を見たときにこんなに簡単に動くゲームが書くことができることにおどろいたため、同じように作ってみようと思ったのが動機です。少しわかりやすく変えています。




# 場の設定
```python
field=[
    ["0","1","2"],
    ["3","4","5"],
    ["6","7","8"]
]
```
```python
field[0]=["0","1","2"]
field[1]=["3","4","5"]
field[2]=["6","7","8"]
```
場を0~8の番号で設定し,数字を選択し,マルバツを入れていく.

二つ目の書き方をすることで条件を作るとき初めに出てくる[x]が何行目を表しているかを理解することができる.ここが自作した部分になる。

# 終了条件の設定
```python    
def judge(field):
    for x in field:
        if x[0]==x[1] and x[1]==x[2]:
            return x[0]
```
ここで一行そろった時にそろった記号を返すように設定する.
```python
    for y in range(3):
        if field[0][y]==field[1][y] and field[1][y]==field[2][y]:
            return[0][y]
```
列をrangeとし,列がそろった時にさっきと同じようにそろった記号を返すようにしている.
```python
    if field[0][0]==field[1][1] and field[1][1]==field[2][2]:
        return field[0][0]
    if field[0][2]==field[1][1] and field[1][1]== field[2][0]:
        return field[0][2]
```
ここは無理やり斜めがそろった時にそろった記号を返すようにしている.



       
# 数字を丸に変更
```python        
int0_space=False
for i in range(3):
    for j in range(3):
        if int0==field[i][j]:
        field[i][j]="o"
        int0_space = True
                    
 if not int0_space:
    print("can't choose")
    continue
```
数字を入れてもらいそこに記号が入っていないか確認させる.確認して入っていればもう一度入れるように要求する.

# 相手の行動
```python
    while True:
        x=random.randint(0,2)
        y=random.randint(0,2)
        
        if field[x][y]!="o" and field[x][y]!="x":
            field[x][y]="x"
            break
```
ランダムで0から2の数字を取ってやり,場を座標とし,何行何列に数字が入るかを決めている.ただ,そこに〇が入っているとダメなのでifでそれを解決している.

#　勝敗の決定
## 勝利
```python
    for i in field:
        print(i)
    if judge(field)=="o":
        print("you win")
        break
```
先ほど終了条件として設定したどれかがそろっている,かつ〇がそろっている場合勝利となる.


## 引き分け
```python
    turn += 1
    if turn==5:
        print("draw")
        break
```
ターン数を一回ずつ増やしていき,自分のターンが5回おわるとすべてのマスが埋まるので,そこで引き分けとする.


## 負け
```python
    for i in field:
        print(i)
    if judge(field)=="x":
        print("you lose")
```
これは勝利の時とほぼ同じだが,違う点としては×がそろっているときというところだけである.

# ○を表示
```python
        int0 = input("where")
        
        int0_space=False
        for i in range(3):
            for j in range(3):
                if int0==field[i][j]:
                    field[i][j]="o"
                    select_space = True
                    
        if not int0_space:
            print("can't choose")
            continue
```
ここでは自分のターンの記号を代入するコードを書いている.
元のものではint0のところはselectだったのですが,数字を入れるためintに変更している.

# 元のコードとの変更点
ゲーム流れを理解しやすいようにタイムリープをつけていましたが,今回はtime.sleep(1)を削りました.そのためimport timeも省略.




# ソースコード
```
import random

from numpy import int0




field=[
    ["0","1","2"],
    ["3","4","5"],
    ["6","7","8"]
]

field[0]=["0","1","2"]
field[1]=["3","4","5"]
field[2]=["6","7","8"]


for i in field:
    print(i)
    
def judge(field):
    for x in field:
        if x[0]==x[1] and x[1]==x[2]:
            return x[0]
    for y in range(3):
        if field[0][y]==field[1][y] and field[1][y]==field[2][y]:
            return[0][y]
    if field[0][0]==field[1][1] and field[1][1]==field[2][2]:
        return field[0][0]
    if field[0][2]==field[1][1] and field[1][1]== field[2][0]:
        return field[0][2]
    return 0


turn=0
while True:
    print("------------")
    for i in field:
        print(i)
       
        int0 = input("where")
        
        int0_space=False
        for i in range(3):
            for j in range(3):
                if int0==field[i][j]:
                    field[i][j]="o"
                    select_space = True
                    
        if not int0_space:
            print("can't choose")
            continue
    
    for i in field:
        print(i)
    if judge(field)=="o":
        print("you win")
        break
    
    turn += 1
    if turn==5:
        print("draw")
        break
    
    print("change turn")
    
    while True:
        x=random.randint(0,2)
        y=random.randint(0,2)
        
        if field[x][y]!="o" and field[x][y]!="x":
            field[x][y]="x"
            break
        
    for i in field:
        print(i)
    if judge(field)=="x":
        print("you lose")
        break
```